//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|                                                  WeisWave3.1.mq4 |
//|         This code comes as is and carries NO WARRANTY whatsoever |
//|                                            Use at your own risk! |
//| Originted: http://www.forexfactory.com/wolfsch                   |
//| http://www.forexfactory.com/showthread.php?p=8412902#post8412902 |
//+------------------------------------------------------------------+
#property strict
#property indicator_separate_window
#property indicator_minimum 0
#property indicator_buffers 2
#property indicator_plots   2
//--- plot upVolume
#property indicator_label1  "upVolume"
#property indicator_type1   DRAW_HISTOGRAM
#property indicator_color1  clrOrange
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2
//--- plot dnVolume
#property indicator_label2  "dnVolume"
#property indicator_type2   DRAW_HISTOGRAM
#property indicator_color2  clrRed
#property indicator_style2  STYLE_SOLID
#property indicator_width2  2
//--- indicator constants
const string short_name = "WeisWave3_3";
const int CHARTWINDOW   = 0;
const int NOSHIFT       = 0;
const int LONGWAVE      = 1;
const int SHORTWAVE     = -1;
const int NOWAVE        = 0;
const int NOVOLUME      = 0;
const string uniquifier = "847";
//--- input parameters
input int      M1_Difference    = 0;          // Enter zero to calculate as 2* 200 period ATR (30 might be reasonable)
input int      M5_Difference    = 0;          // Enter zero to calculate as 2* 200 period ATR (60 might be reasonable)
input int      M15_Difference   = 0;          // Enter zero to calculate as 2* 200 period ATR 140 might be reasonable)
input int      M30_Difference   = 0;          // Enter zero to calculate as 2* 200 period ATR (200 might be reasonable)
input int      H1_Difference    = 0;          // Enter zero to calculate as 2* 200 period ATR (265 might be reasonable)
input int      H4_Difference    = 0;          // Enter zero to calculate as 2* 200 period ATR (500 might be reasonable)
input int      D1_Difference    = 0;          // Enter zero to calculate as 2* 200 period ATR (1450 might be reasonable)
input int      W1_Difference    = 0;          // Enter zero to calculate as 2* 200 period ATR (4400 might be reasonable)
input int      MN1_Difference   = 0;          // Enter zero to calculate as 2* 200 period ATR (1100 might be reasonable)
input bool     showWaveLines    = true;       // Show line from LL to HH for each wave
input bool     showChangePrices = true;       // Put a price mark where the wave change occurs
input bool     ShowVolumeLabels = true;       // Show the volume amount on the chart
input color    LongColor        = clrOrange;
input color    ShortColor       = clrRed;
input int      WaveWidth        = 2;
//--- indicator buffers
double         upVolumeBuffer[];
double         dnVolumeBuffer[];
double         barDirection[];
double         trendDirection[];
double         waveDirection[];
long           barTickVolume = 0;
//
enum TF_INDEX { m1Idx, m5Idx, m15Idx, m30Idx, h1Idx, h4Idx, d1Idx, w1Idx, mn1Idx, endTFIdx};
int ary_TFdifference[endTFIdx];

double         highestHigh = EMPTY_VALUE;
double         lowestLow   = EMPTY_VALUE;
int            hhBar       = EMPTY_VALUE;
int            llBar       = EMPTY_VALUE;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
 {
   // Set the Wave Change difference value
   ary_TFdifference[m1Idx]  = M1_Difference;
   ary_TFdifference[m5Idx]  = M5_Difference;  
   ary_TFdifference[m15Idx] = M15_Difference;   
   ary_TFdifference[m30Idx] = M30_Difference;
   ary_TFdifference[h1Idx]  = H1_Difference;
   ary_TFdifference[h4Idx]  = H4_Difference;
   ary_TFdifference[d1Idx]  = D1_Difference;
   ary_TFdifference[w1Idx]  = W1_Difference;
   ary_TFdifference[mn1Idx] = MN1_Difference;
   TF_INDEX CURRENT_TF_IDX = GetCurrentPeriodIDX();
   if( ary_TFdifference[CURRENT_TF_IDX] < 1 )  // no valid value supplied
    {
      ary_TFdifference[CURRENT_TF_IDX] = (int) (2*(iATR(Symbol(),PERIOD_CURRENT,200,NOSHIFT)/Point));
    }
   //
   IndicatorShortName(short_name+" "+IntegerToString(ary_TFdifference[CURRENT_TF_IDX]));
   //
   //--- indicator buffers mapping
   IndicatorBuffers(5);
   SetIndexBuffer(0, upVolumeBuffer);
   SetIndexEmptyValue(0, EMPTY_VALUE);
   SetIndexBuffer(1, dnVolumeBuffer);
   SetIndexEmptyValue(1, EMPTY_VALUE);
   SetIndexBuffer(2, barDirection);
   SetIndexEmptyValue(2, NOWAVE);
   SetIndexBuffer(3, trendDirection);
   SetIndexEmptyValue(3, NOWAVE);
   SetIndexBuffer(4, waveDirection);
   SetIndexEmptyValue(4, NOWAVE);
   //---
   return(INIT_SUCCEEDED);
 }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
 {
   int compareStrLen = StringLen(uniquifier);
   for(int i = ObjectsTotal() - 1; i >= 0; i--)
     {
       string Label = ObjectName(i);
       if (StringCompare(uniquifier, StringSubstr(Label, 0, compareStrLen), true) == 0)  // StringCompare returns zero on match
        {
         ObjectDelete(Label);
        }
     }
 }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total, const int prev_calculated, const datetime &time[], const double &open[], const double &high[],
                const double &low[], const double &close[], const long& tick_volume[], const long& volume[], const int& spread[] )
 {
   if (rates_total == prev_calculated)
    { return(rates_total); }  // Only compute bars on new bar
   int limit;
   int waveChangeBar;
   static string waveID;
   if ( (prev_calculated > 0) && ( (upVolumeBuffer[(rates_total-prev_calculated)+1] != EMPTY_VALUE) || (dnVolumeBuffer[(rates_total-prev_calculated)+1] != EMPTY_VALUE) ) ) // A wave direction is already set
    {  // A wave direction is already set,  Initialize values  waveID, HighestHigh, LowestLow, hhbar and llbar all all global (could be made local static) so should have durrent values ready to go.
      limit = (rates_total-prev_calculated)+1;
      waveChangeBar = limit - 1;
    }
   else  // First Time, Initialize values
    {
      limit         = rates_total - 1;
      waveChangeBar = limit - 1;
      waveID = uniquifier + TimeToString(time[waveChangeBar], TIME_DATE|TIME_MINUTES) + "-TL";
      highestHigh   = close[waveChangeBar];
      lowestLow     = close[waveChangeBar];
      hhBar         = waveChangeBar;
      llBar         = waveChangeBar;
    }
   //
   for(int i=limit-1; i>=0; i--)
    {
      // Determine this bar's direction
      if (close[i] - close[i+1] >  0) barDirection[i] =  LONGWAVE;     // current close higher
      if (close[i] - close[i+1] == 0) barDirection[i] =  NOWAVE;       // current close equal
      if (close[i] - close[i+1] <  0) barDirection[i] =  SHORTWAVE;    // current close lower

      if (barDirection[limit]   == NOWAVE) barDirection[limit]   = barDirection[i];
      if (trendDirection[limit] == NOWAVE) trendDirection[limit] = barDirection[i];
      if (waveDirection[limit]  == NOWAVE) waveDirection[limit]  = barDirection[i];

      // Determine highset high and lowest low
      if (close[i] > highestHigh)
       {
         highestHigh = close[i];
         hhBar = i;
       }
      else if (close[i] < lowestLow)
       {
         lowestLow = close[i];
         llBar = i;
       }
      // Determine if this bar has started a new trend
      if ((barDirection[i] != 0) && (barDirection[i] != barDirection[i+1]))
       { trendDirection[i] = barDirection[i]; }
      else 
       { trendDirection[i] = trendDirection[i+1]; }

      // Determine if this bar has started a new wave
      double waveTest = 0.0;
      if (waveDirection[i+1] == LONGWAVE)
       {
         waveTest = highestHigh;
       }
      if (waveDirection[i+1] == SHORTWAVE)
       {
         waveTest = lowestLow;
       }
      double waveDifference = (MathAbs(waveTest - close[i])) * MathPow(10, Digits);
      if (trendDirection[i] != waveDirection[i+1])
       {
         if (waveDifference >= ary_TFdifference[GetCurrentPeriodIDX()]) waveDirection[i] = trendDirection[i];
         else waveDirection[i] = waveDirection[i+1];
       }
      else
       { waveDirection[i] = waveDirection[i+1]; }

      // Determine if we have started a new wave
      if (waveDirection[i] != waveDirection[i+1])
       {
         if (waveDirection[i] == LONGWAVE)
          {
            highestHigh = close[i];
            hhBar = i;
            waveChangeBar = llBar;
          }
         else   // == SHORTWAVE
          {
            lowestLow = close[i];
            llBar = i;
            waveChangeBar = hhBar;
          }
         //
         string saveName = waveID;
         waveID = MakeWaveName( waveChangeBar );  // Make a new bave trend line
         if(showWaveLines)
          {
            UpdateWaveEnd(saveName, time[waveChangeBar], close[waveChangeBar]);  // Final trend line Wave End Update
            DisplayWave(waveID, time[waveChangeBar], close[waveChangeBar], time[i], close[i], (int) waveDirection[i]);
          }
         if (showChangePrices)
          {
            static int callCount = 0;  callCount++;
            DispalyNameChangeMark(saveName, waveID, i+1, (int) waveDirection[i], callCount);
          }
         //
         barTickVolume = 0;
         for (int k=waveChangeBar-1; k>=i; k-- )
          {
            barTickVolume += tick_volume[k];
            if (waveDirection[i] ==  LONGWAVE)
             {
               upVolumeBuffer[k] = (double) barTickVolume;
               dnVolumeBuffer[k] = NOVOLUME;
             }
            if (waveDirection[i] == SHORTWAVE)
             {
               upVolumeBuffer[k] = NOVOLUME;
               dnVolumeBuffer[k] = (double) barTickVolume;
             }
          }
         //
         if (ShowVolumeLabels == true)
          {
            string volLabel = uniquifier + TimeToString(time[waveChangeBar], TIME_DATE|TIME_MINUTES) + "-VOL";
            if (waveDirection[i] == LONGWAVE)
             {
               string displayText = DoubleToString(dnVolumeBuffer[waveChangeBar], 0);
               WaveLineVolLabel(waveChangeBar, displayText);
             }
            else   // SHORTWAVE
             {
               string displayText = DoubleToString(upVolumeBuffer[waveChangeBar], 0);
               WaveLineVolLabel(waveChangeBar, displayText);
             }
          }
       }
      else
       {
         barTickVolume += tick_volume[i];
       }
      // Set the indicators
      if (waveDirection[i] ==  LONGWAVE )
       {
         upVolumeBuffer[i] = (double) barTickVolume;
         dnVolumeBuffer[i] = NOVOLUME;
       }
      if (waveDirection[i] == SHORTWAVE)
       {
         upVolumeBuffer[i] = NOVOLUME;
         dnVolumeBuffer[i] = (double) barTickVolume;
       }
    }
   //
   if(showWaveLines)
    {
      UpdateWaveEnd(waveID, time[0], close[0]);
    }
   //--- return value of prev_calculated for next call
   return(rates_total);
 }
//+------------------------------------------------------------------+
void DispalyNameChangeMark(string saveName, string newName, int changeBar, int direction, int callCount)
 {
   static int lastCallCount = 0;
   string priceName = uniquifier+"waveChangeMark"+saveName;
   string textName  = uniquifier+"textChangeMark";  // there will only be one of these
   long chartID = ChartID();
   if( ObjectFind(priceName) < 0 ) // <0 is 'not found'
    {
      ObjectCreate(chartID,priceName,direction==LONGWAVE?OBJ_ARROW_LEFT_PRICE:OBJ_ARROW_RIGHT_PRICE,CHARTWINDOW,Time[changeBar],direction==LONGWAVE?High[changeBar]+(Ask-Bid):Low[changeBar]); 
      ObjectSetInteger(chartID,priceName,OBJPROP_COLOR,direction==LONGWAVE?ShortColor:LongColor); //  Show with the color of the ending line
      ObjectSetInteger(chartID,priceName,OBJPROP_STYLE,STYLE_SOLID); 
      ObjectSetInteger(chartID,priceName,OBJPROP_WIDTH,3); 
      ObjectSetInteger(chartID,priceName,OBJPROP_BACK,FALSE);  
      ObjectSetInteger(chartID,priceName,OBJPROP_SELECTABLE,false); 
      ObjectSetInteger(chartID,priceName,OBJPROP_SELECTED,false); 
      ObjectSetInteger(chartID,priceName,OBJPROP_HIDDEN,true);
    }
   else
    {
      ObjectMove(chartID,priceName,CHARTWINDOW,Time[changeBar],direction==LONGWAVE?High[changeBar]+(Ask-Bid):Low[changeBar]);
    }
   /*
   if( ObjectFind(textName) < 0 ) // <0 is 'not found'
    {
      ObjectCreate(chartID, textName, OBJ_TEXT,CHARTWINDOW, TimeCurrent(), WindowPriceMax(CHARTWINDOW)-(Ask-Bid));
      ObjectSet(textName, OBJPROP_ANCHOR, ANCHOR_RIGHT_UPPER);
      ObjectSetText(textName, saveName+" to "+newName+", "+IntegerToString(callCount-lastCallCount)+", "+IntegerToString(callCount), 10, NULL, clrOrange); 
      ObjectSetInteger(chartID,textName,OBJPROP_BACK,false);  
      ObjectSetInteger(chartID,textName,OBJPROP_SELECTABLE,false); 
      ObjectSetInteger(chartID,textName,OBJPROP_SELECTED,false); 
      ObjectSetInteger(chartID,textName,OBJPROP_HIDDEN,true);
    }
   else
    {
      ObjectMove(chartID,textName,CHARTWINDOW,TimeCurrent(),WindowPriceMax(CHARTWINDOW)-(Ask-Bid));
      ObjectSetText(textName,saveName+" to "+newName+", "+IntegerToString(callCount-lastCallCount)+", "+IntegerToString(callCount), 10, NULL, clrOrange);
    }
   */
   lastCallCount = callCount;
 }
//+------------------------------------------------------------------+
string MakeWaveName( int waveStartBar )
 {
   return(uniquifier+"|"+TimeToString(Time[waveStartBar], TIME_DATE|TIME_MINUTES) + "-TL");
 }
//
void DisplayWave(string waveName, datetime startTime, double startPrice, datetime endTime, double endPrice,int lineDirection)
 { 
   long chartID = ChartID();
   if ( ObjectFind(chartID,waveName) < 0 ) // No object found
    {
      ObjectCreate(chartID, waveName, OBJ_TREND, CHARTWINDOW, startTime, startPrice, endTime, endPrice);
      ObjectSet(waveName, OBJPROP_COLOR, lineDirection==LONGWAVE?LongColor:ShortColor);
      ObjectSet(waveName, OBJPROP_RAY, false);
      ObjectSet(waveName, OBJPROP_WIDTH, 2);
      ObjectSetInteger(chartID,waveName,OBJPROP_BACK,false);  
      ObjectSetInteger(chartID,waveName,OBJPROP_SELECTABLE,false); 
      ObjectSetInteger(chartID,waveName,OBJPROP_SELECTED,false); 
      ObjectSetInteger(chartID,waveName,OBJPROP_HIDDEN,true);
    }
   else
    {
      UpdateWaveEnd(waveName, endTime, endPrice);
    }
 }
//
void UpdateWaveEnd(string waveName, datetime endTime, double endPrice)
 {
    ObjectSet(waveName, OBJPROP_PRICE2, endPrice);
    ObjectSet(waveName, OBJPROP_TIME2, endTime);
 }
void WaveLineVolLabel(int waveChangeBar, string displayText)
 {
   long chartID = ChartID();
   string labelName = MakeLabelName( Time[waveChangeBar]);
   if (WindowFind( labelName ) < 0 )
    {
      ObjectCreate(chartID, labelName, OBJ_TEXT, CHARTWINDOW, Time[waveChangeBar], waveDirection[waveChangeBar]==LONGWAVE?High[waveChangeBar]:Low[waveChangeBar]);
      ObjectSet(labelName, OBJPROP_ANGLE, waveDirection[waveChangeBar]==LONGWAVE?90:-90);
      ObjectSet(labelName, OBJPROP_ANCHOR, ANCHOR_LEFT);
      ObjectSetText(labelName, displayText, 10, NULL, waveDirection[waveChangeBar]==LONGWAVE?LongColor:ShortColor);
    }
   else
    {
      ObjectSet(labelName, OBJPROP_PRICE1, waveDirection[waveChangeBar]==LONGWAVE?High[waveChangeBar]:Low[waveChangeBar]);
      ObjectSet(labelName, OBJPROP_TIME1,  Time[waveChangeBar]);
    }
 }
//
string MakeLabelName( datetime waveTime )
 {
   return(uniquifier+" "+TimeToString(waveTime, TIME_DATE|TIME_MINUTES) + "-VOL");
 }
//
TF_INDEX GetCurrentPeriodIDX(void)
 {
   TF_INDEX CURRENT_IDX;
   switch(Period())
   {
      case  PERIOD_M1:
            CURRENT_IDX = m1Idx;
            break;       
      case  PERIOD_M5:
            CURRENT_IDX = m5Idx;
            break;
      case  PERIOD_M15:
            CURRENT_IDX = m15Idx;
            break;      
      case  PERIOD_M30:
            CURRENT_IDX = m30Idx;
            break;      
      case  PERIOD_H1:
            CURRENT_IDX = h1Idx;
            break;      
      case  PERIOD_H4:
            CURRENT_IDX = h4Idx;
            break;       
      case  PERIOD_D1:
            CURRENT_IDX = d1Idx;            
            break;       
      case PERIOD_W1:
            CURRENT_IDX = w1Idx;
            break;       
      case PERIOD_MN1:
            CURRENT_IDX = mn1Idx;
            break;       
      default:
            CURRENT_IDX = m15Idx;
            break;
   }
   return(CURRENT_IDX);
 }
//